#include "ProjectManager.h"
#include "../../core/utils/EngineConfig.h"
#include "../../core/utils/Logging.h"
#include "../../Engine.h"
#include "../../core/utils/FileDialog.h"
#include "../../core/utils/utils.h"
#include "../../core/utils/AssetManager.h"

#include "SceneSerializer.h"

#include <yaml-cpp/yaml.h>
#include <filesystem>
#include <fstream>
#include <chrono>
#include <imgui.h>
#include <string>
#include <cstring>







bool                       FileExplorer::s_initialized   = false;
std::string                FileExplorer::SelectedPath;
fs::path                   FileExplorer::s_root;
fs::path                   FileExplorer::s_currentDir;
std::unordered_set<std::string> FileExplorer::s_knownFiles;
const char*                FileExplorer::ICONS_PATH = "./src/assets/icons/";
unsigned int               FileExplorer::s_folderIcon = 0;

std::string                FileExplorer::fileSearchQuery = "";
bool                       FileExplorer::sortAscending = true;
int                        FileExplorer::sortMode = 0;
bool                       FileExplorer::showScenes = true;
bool                       FileExplorer::showImages = true;
bool                       FileExplorer::showAudio = true;
bool                       FileExplorer::showScripts = true;
bool                       FileExplorer::showOther = true;

std::vector<fs::path>      FileExplorer::s_loadQueue;
size_t                     FileExplorer::s_loadIndex = 0;
bool                       FileExplorer::s_loadingInit = false;


bool FileExplorer::Init()
{
    if (s_initialized)
        return false;

    if (!ProjectManager::HasProject())
        return false;

    // Resolve and verify root…
    s_root = fs::path(ProjectManager::ResolveResPath("res://"));
    s_currentDir = s_root;
    if (!fs::exists(s_root) || !fs::is_directory(s_root)) {
        Logger::LogError("FileExplorer: invalid root path '%s'", s_root.string().c_str());
        return false;
    }

    s_folderIcon = EngineLoadTextureIfNeeded(std::string(ICONS_PATH) + "folder-outline.png");

    std::error_code ec;
    fs::recursive_directory_iterator it(s_root, fs::directory_options::skip_permission_denied, ec);
    fs::recursive_directory_iterator end;
    for (; it != end; it.increment(ec)) {
        if (ec) {
            Logger::LogWarning("FileExplorer: skipping '%s': %s",
                            it->path().string().c_str(),
                            ec.message().c_str());
            ec.clear();
            continue;
        }
        if (it->is_regular_file())
            s_loadQueue.push_back(it->path());
    }

    s_initialized = true;
    return true;
}



void FileExplorer::Show(bool* p_open)
{
    if (!s_initialized) return;
    if (!ImGui::Begin("File Explorer", p_open,
            ImGuiWindowFlags_NoCollapse | ImGuiWindowFlags_HorizontalScrollbar))
    {
        ImGui::End();
        return;
    }

    // --- Left pane: folder tree ---
    ImGui::BeginChild("##FolderPane", ImVec2(250, 0), true);
      ImGui::TextUnformatted("res://");
      ImGui::Separator();
      DrawFolderTree(s_root);
    ImGui::EndChild();

    ImGui::SameLine();

    // --- Right pane: content + toolbar ---
    ImGui::BeginChild("##ContentPane", ImVec2(0, 0), false);

    // Filters / Search
    if (ImGui::Button("Filters / Sort")) ImGui::OpenPopup("FileFilterPopup");
    ImGui::SameLine();
    ImGui::SetNextItemWidth(200);
    char buf[256];
    strncpy(buf, fileSearchQuery.c_str(), sizeof(buf));
    buf[sizeof(buf)-1] = '\0';
    if (ImGui::InputTextWithHint("##Search", "Search...", buf, sizeof(buf)))
        fileSearchQuery = buf;

    if (ImGui::BeginPopup("FileFilterPopup")) {
        ImGui::Text("Sort By:");
        if (ImGui::Selectable("Name", sortMode == 0)) sortMode = 0;
        if (ImGui::Selectable("Type", sortMode == 1)) sortMode = 1;
        ImGui::Checkbox("Ascending", &sortAscending);
        ImGui::Separator();
        ImGui::Text("Type:");
        ImGui::Checkbox("Scenes", &showScenes);
        ImGui::Checkbox("Images", &showImages);
        ImGui::Checkbox("Audio",  &showAudio);
        ImGui::Checkbox("Scripts", &showScripts);
        ImGui::Checkbox("Other",   &showOther);
        ImGui::EndPopup();
    }

    ImGui::Separator();

    // “Up” button
    if (s_currentDir != s_root) {
        if (ImGui::Button("Up")) s_currentDir = s_currentDir.parent_path();
        ImGui::SameLine();
    }

    // Current path display
    fs::path rel = fs::relative(s_currentDir, s_root);
    std::string displayPath = "res://" +
        (rel.empty() ? std::string() : rel.generic_string() + "/");
    ImGui::TextUnformatted(displayPath.c_str());
    ImGui::Separator();

    // Gather & filter
    std::vector<fs::directory_entry> entries;
    for (auto &e : fs::directory_iterator(s_currentDir)) {
        if (!e.is_directory()) {
            AssetType t = AssetManager::AssetTypeFromPath(e.path().string());
            switch (t) {
                case AssetType::Scene:  if (!showScenes)  continue; break;
                case AssetType::Image:  if (!showImages)  continue; break;
                case AssetType::Audio:  if (!showAudio)   continue; break;
                case AssetType::Script: if (!showScripts) continue; break;
                default:                if (!showOther)   continue; break;
            }
        }
        if (!fileSearchQuery.empty() &&
            e.path().filename().string().find(fileSearchQuery) == std::string::npos)
            continue;
        entries.push_back(e);
    }

    // Sort
    std::sort(entries.begin(), entries.end(), [&](auto &a, auto &b) {
        if (sortMode == 1) {
            bool da = a.is_directory(), db = b.is_directory();
            if (da != db) return sortAscending ? da > db : da < db;
            return sortAscending
                ? a.path().extension() < b.path().extension()
                : a.path().extension() > b.path().extension();
        }
        return sortAscending
            ? a.path().filename() < b.path().filename()
            : a.path().filename() > b.path().filename();
    });

    // Icon grid setup
    constexpr float iconSize = 64.0f, padding = 8.0f;
    float avail = ImGui::GetContentRegionAvail().x;
    int cols = std::max(1, int(avail / (iconSize + padding)));
    ImGui::Columns(cols, nullptr, false);

    // Draw each entry
    for (auto &e : entries) {
        fs::path p = e.path();
        std::string pathStr = p.string();
        AssetType type = AssetManager::AssetTypeFromPath(pathStr);

        ImGui::PushID(pathStr.c_str());

        // --- IMAGE THUMBNAIL w/ InvisibleButton for clicks ---
        if (type == AssetType::Image) {
            if (auto* asset = AssetManager::GetAssetByPath(pathStr))
            if (asset->type == AssetType::Image) {
                auto* img = dynamic_cast<const ImageAssetInfo*>(asset);
                if (img) {
                    if (ImGui::ImageButton("##Image", (ImTextureID)(intptr_t)img->textureID, ImVec2(iconSize, iconSize)))
                    {
                        s_currentDir = p;
                    }
                }
            }
        }
        else if (e.is_directory()) {
            if (ImGui::ImageButton("##item",
                (ImTextureID)(intptr_t)s_folderIcon,
                ImVec2(iconSize, iconSize)))
            {
                s_currentDir = p;
            }
        }
        else {
            std::string label;
            switch (type) {
              case AssetType::Prefab:  label = "Prefab"; break;
              case AssetType::Scene:   label = "Scene";  break;
              case AssetType::Audio:   label = "Audio";  break;
              case AssetType::Script:  label = "Script"; break;
              case AssetType::Video:   label = "Video";  break;
              case AssetType::Font:    label = "Font";   break;
              case AssetType::Shader:  label = "Shader"; break;
              default:
                label = p.extension().string();
                if (!label.empty() && label.front()=='.')
                    label.erase(0,1);
                break;
            }
            if (ImGui::Button(label.c_str(),
                              ImVec2(iconSize, iconSize)))
            {
                SelectedPath = pathStr;
            }
        }

        if (ImGui::BeginDragDropSource(ImGuiDragDropFlags_SourceAllowNullID)) {
            if (auto* asset = AssetManager::GetAssetByPath(pathStr)) {
                if (type == AssetType::Image)
                    ImGui::SetDragDropPayload(ASSET_TEXTURE,
                                              &asset->uaid,
                                              sizeof(asset->uaid));
                else if (type == AssetType::Prefab)
                    ImGui::SetDragDropPayload(ASSET_PREFAB,
                                              &asset->uaid,
                                              sizeof(asset->uaid));
                ImGui::TextUnformatted(p.filename().string().c_str());
            }
            ImGui::EndDragDropSource();
        }

        if (ImGui::BeginPopupContextItem()) {
            if (e.is_directory() && ImGui::MenuItem("Open"))
                s_currentDir = p;
            if (!e.is_directory() && ImGui::MenuItem("Open File"))
                SelectedPath = pathStr;
            if (ImGui::MenuItem("Copy Path"))
                ImGui::SetClipboardText(pathStr.c_str());
            ImGui::EndPopup();
        }

        ImGui::TextWrapped("%s", p.filename().string().c_str());
        ImGui::NextColumn();
        ImGui::PopID();
    }

    ImGui::Columns(1);
    ImGui::EndChild();
    ImGui::End();
}




void FileExplorer::Update()
{
    if (!s_initialized)
    {
        Init();
        return;
    }

    std::error_code ec;
    std::unordered_set<std::string> currentFiles;
    for (auto it = fs::recursive_directory_iterator(s_root,
                fs::directory_options::skip_permission_denied, ec),
         end = fs::recursive_directory_iterator{};
         it != end; it.increment(ec))
    {
        if (ec) {
            Logger::LogWarning("FileExplorer: skipping '%s': %s",
                            it->path().string().c_str(),
                            ec.message().c_str());
            ec.clear();
            continue;
        }
        if (!it->is_regular_file()) continue;

        auto pathStr = it->path().string();
        currentFiles.insert(pathStr);

        if (!s_knownFiles.count(pathStr)) {
            s_knownFiles.insert(pathStr);
            s_loadQueue.push_back(it->path());
            Logger::LogVerbose("FileExplorer: added '%s'", pathStr.c_str());
        }
    }

    // detect deletions
    for (auto it = s_knownFiles.begin(); it != s_knownFiles.end(); )
    {
        if (!currentFiles.count(*it)) {
            Logger::LogVerbose("FileExplorer: removed '%s'", it->c_str());
            AssetManager::GetAssetByPath(*it);
            it = s_knownFiles.erase(it);
        }
        else ++it;
    }
}

bool FileExplorer::LoadingDone()
{
    return s_initialized && s_loadIndex >= s_loadQueue.size();
}

void FileExplorer::DrawFolderTree(const fs::path &path)
{
    for (auto &e : fs::directory_iterator(path)) {
        if (!e.is_directory()) continue;
        auto &d = e.path();
        ImGui::PushID(d.string().c_str());
        ImGui::Image((ImTextureID)(intptr_t)s_folderIcon, ImVec2(16,16)); ImGui::SameLine();
        bool open = ImGui::TreeNodeEx(d.string().c_str(), ImGuiTreeNodeFlags_OpenOnArrow | ImGuiTreeNodeFlags_SpanAvailWidth, "%s", d.filename().string().c_str());
        if (ImGui::IsItemClicked()) s_currentDir = d;
        if (open) { DrawFolderTree(d); ImGui::TreePop(); }
        ImGui::PopID();
    }
}

void FileExplorer::ProcessLoadQueue()
{
    while (!s_loadQueue.empty()) {
        fs::path p = s_loadQueue.back();
        s_loadQueue.pop_back();

        // Load the asset and get the type,...  Long way...
        AssetManager::LoadAssetAsync(p.string(), AssetManager::AssetTypeFromExtension(AssetManager::GetFileExtension(p.string())));

        Logger::LogVerbose("FileExplorer: imported '%s'", p.string().c_str());
    }
}


std::string FileExplorer::IconFileForPath(const fs::path &path)
{
    auto ext = path.extension().string();
    if (ext == ".cene") return "movie-open-outline.png";
    return "error.png";
}



namespace fs = std::filesystem;

std::string ProjectManager::s_currentProjectPath;
std::string ProjectManager::s_currentProjectName;
std::string ProjectManager::s_defaultScene;




void ProjectManager::ShowCreateProjectPopup()
{
    static char nameBuf[256] = "";
    static char pathBuf[512] = "";

    ImGui::SetNextWindowSizeConstraints({ 0, 0 }, { 600, FLT_MAX });
    if (!ImGui::BeginPopupModal(
            "Create New Project",
            nullptr,
            ImGuiWindowFlags_AlwaysAutoResize))
        return;

    ImGui::Text("Create New Project");
    ImGui::Separator();
    ImGui::Spacing();

    // Project Name (full width)
    ImGui::TextUnformatted("Project Name");
    ImGui::PushItemWidth(-1);
    ImGui::InputText("##proj_name", nameBuf, IM_ARRAYSIZE(nameBuf));
    ImGui::PopItemWidth();
    ImGui::Spacing();

    ImGui::TextUnformatted("Project Folder");
    ImGui::PushItemWidth(-85);
    ImGui::InputText("##proj_path", pathBuf, IM_ARRAYSIZE(pathBuf));
    ImGui::PopItemWidth();
    ImGui::SameLine();
    if (ImGui::Button("Browse...", { 80, 0 }))
    {
        if (auto chosen = OpenFolderDialog(); !chosen.empty())
            std::strncpy(pathBuf, chosen.c_str(), sizeof(pathBuf));
    }
    ImGui::Spacing();

    // Preview path – wrapped so it never overflows
    auto preview = std::filesystem::path(pathBuf) / (std::string(nameBuf) + ".cproj");
    ImGui::Text("Will be created at:");
    ImGui::TextWrapped("%s", preview.string().c_str());
    ImGui::Spacing();

    // Action buttons centered
    ImGui::Separator();
    float btnW    = 80.0f;
    float totalW  = btnW * 2 + ImGui::GetStyle().ItemSpacing.x;
    float offsetX = (ImGui::GetWindowWidth() - totalW) * 0.5f;
    ImGui::SetCursorPosX(offsetX);

    if (ImGui::Button("Cancel", { btnW, 0 }))
        ImGui::CloseCurrentPopup();

    ImGui::SameLine();
    ImGui::SetItemDefaultFocus();
    if (ImGui::Button("Create", { btnW, 0 }))
    {
        CreateProject(pathBuf, nameBuf);
        nameBuf[0] = pathBuf[0] = '\0';
        ImGui::CloseCurrentPopup();
    }

    ImGui::EndPopup();
}








bool CreateDirectories(const fs::path &baseDir)
{
    static const std::vector<std::string> subdirs = {
        "Prefabs",
        "Scenes",
        "Images",
        "Sounds",
        "Fonts",
        "Shaders",
        "Scripts",
        "Videos",

    };

    std::error_code ec;
    bool allOk = true;
    fs::path assetsRoot = baseDir / "Assets";

    for (const auto &name : subdirs)
    {
        Logger::LogVerbose("Creating Folder: '%s'", name.c_str());
        fs::create_directories(assetsRoot / name, ec);
        if (ec)
        {
            Logger::LogError("Could not ensure %s dir: %s",
                             name.c_str(),
                             ec.message().c_str());
            allOk = false;
        }
    }

    return allOk;
}

bool ProjectManager::HasProject() {
    if (s_currentProjectPath.empty() || s_currentProjectName.empty())
    {
        return false;

    }
    return true;
}



std::string ProjectManager::ResolveResPath(const std::string &resPath)
{
    constexpr const char *prefix = "res://";
    // if it’s not a “res://” path, just return it
    if (resPath.rfind(prefix, 0) != 0)
        return resPath;

    if (s_currentProjectPath.empty())
    {
        Logger::LogError("Cannot resolve res path: project not loaded!");
        return resPath;
    }

    // start from whatever s_currentProjectPath holds
    fs::path baseDir{s_currentProjectPath};

    // only append the project name if it isn't already the last path component
    if (!s_currentProjectName.empty() &&
        baseDir.filename().string() != s_currentProjectName)
    {
        baseDir /= s_currentProjectName;
    }

    // drop the "res://" prefix
    std::string relativePart = resPath.substr(strlen(prefix));
    // build & normalize
    fs::path fullPath = (baseDir / relativePart).lexically_normal();

    return fullPath.string();
}


bool ProjectManager::Init()
{
    s_currentProjectPath.clear();
    s_currentProjectName.clear();
    s_defaultScene.clear();
    Logger::LogOk("Project Core");
    return true;
}

bool ProjectManager::LoadProject(const std::string &projectFilePath)
{
    fs::path projFile{ projectFilePath };
    if (projFile.extension() != ".cproj" || !fs::exists(projFile)) {
        Logger::LogError("Project file not found or invalid: '%s'",
                         projectFilePath.c_str());
        return false;
    }

    fs::path baseDir   = projFile.parent_path();
    std::string projectName = projFile.stem().string();

    YAML::Node config;
    try {
        config = YAML::LoadFile(projFile.string());
    }
    catch (const YAML::Exception &e) {
        Logger::LogError("Failed to parse project file '%s': %s",
                         projFile.string().c_str(), e.what());
        return false;
    }

    // Read settings
    std::string savedName    = config["Name"].as<std::string>(projectName);
    std::string defaultScene = config["s_defaultScene"].as<std::string>("");

    Logger::LogInfo("Loading Project: '%s' from '%s'",
                    savedName.c_str(), projFile.string().c_str());

    s_currentProjectPath = baseDir.string();
    s_currentProjectName = savedName;

    if (!defaultScene.empty()) {
        s_defaultScene = ResolveResPath(defaultScene);
        Logger::LogDebug("Loading Default Scene: %s", s_defaultScene.c_str());

        if (!fs::exists(s_defaultScene)) {
            Logger::LogError("Default scene does not exist: %s", s_defaultScene.c_str());
            return false;
        }

        SceneLoader::LoadScene(s_defaultScene);
    }

    Logger::LogInfo("Successfully loaded project '%s'", savedName.c_str());

    CreateDirectories(baseDir);

    SceneLoader::LoadScene(s_defaultScene);

    return true;
}


bool ProjectManager::CreateProject(const std::string &projectPath,
                                   const std::string &projectName)
{
    fs::path baseDir = fs::path(projectPath) / projectName;
    fs::path projFile = baseDir / (projectName + ".cproj");

    std::error_code ec;
    if (!fs::create_directories(baseDir, ec) && ec)
    {
        Logger::LogError("Failed to create project directory '%s': %s",
                         projectPath.c_str(), ec.message().c_str());
        return false;
    }

    s_currentProjectPath = baseDir.string();
    s_currentProjectName = projectName;

    fs::path scenePath = baseDir / "Assets" / "Scenes" / projectName / ".cene";


    s_defaultScene = scenePath.string();

    // Write .cproj YAML
    YAML::Emitter out;
    out << YAML::BeginMap;
    out << YAML::Key << "Name" << YAML::Value << projectName;
    out << YAML::Key << "EngineVersion" << YAML::Value << g_engineConfig.version;
    fs::path relPath = fs::relative(scenePath, baseDir);
    out << YAML::Key << "s_defaultScene" << YAML::Value << ("res://" + relPath.generic_string());

    auto now = std::chrono::system_clock::now();
    auto t = std::chrono::system_clock::to_time_t(now);
    std::tm tm{};
#if defined(_WIN32)
    gmtime_s(&tm, &t);
#else
    gmtime_r(&t, &tm);
#endif
    char buf[32];
    std::strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", &tm);
    out << YAML::Key << "CreatedDate" << YAML::Value << buf;
    out << YAML::EndMap;

    std::ofstream fout(projFile);
    if (!fout.is_open())
    {
        Logger::LogError("Could not open project file for writing: %s",
                         projFile.string().c_str());
        return false;
    }
    fout << out.c_str();
    fout.close();

    Logger::LogOk("Created project '%s' at '%s'",
                  projectName.c_str(), projectPath.c_str());

    CreateDirectories(baseDir);

    SceneLoader::SaveScene(scenePath.string());


    SceneLoader::LoadScene(scenePath.string());

    return true;
}


bool ProjectManager::SaveProject(const std::string &projectPath,
                                 const std::string &projectName)
{
    fs::path baseDir = fs::path(projectPath) / projectName;
    fs::path projFile = baseDir / (projectName + ".cproj");

    std::error_code ec;
    if (!fs::create_directories(baseDir, ec) && ec)
    {
        Logger::LogError("Failed to create project directory '%s': %s",
                         projectPath.c_str(), ec.message().c_str());
        return false;
    }



    YAML::Emitter out;
    out << YAML::BeginMap;
    out << YAML::Key << "Name" << YAML::Value << projectName;
    out << YAML::Key << "EngineVersion" << YAML::Value << g_engineConfig.version;
    fs::path relPath = fs::relative(s_defaultScene,
                                    fs::path(s_currentProjectPath) / s_currentProjectName);
    out << YAML::Key << "s_defaultScene" << YAML::Value << ("res://" + relPath.generic_string());

    auto now = std::chrono::system_clock::now();
    auto t = std::chrono::system_clock::to_time_t(now);
    std::tm tm{};
#if defined(_WIN32)
    gmtime_s(&tm, &t);
#else
    gmtime_r(&t, &tm);
#endif
    char buf[32];
    std::strftime(buf, sizeof(buf), "%Y-%m-%dT%H:%M:%SZ", &tm);
    out << YAML::Key << "CreatedDate" << YAML::Value << buf;
    out << YAML::EndMap;

    std::ofstream fout(projFile);
    if (!fout.is_open())
    {
        Logger::LogError("Could not open project file for writing: %s",
                         projFile.string().c_str());
        return false;
    }
    fout << out.c_str();
    fout.close();

    Logger::LogOk("Saved project '%s' at '%s'",
                  projectName.c_str(), projectPath.c_str());

    CreateDirectories(baseDir);

    s_currentProjectPath = projectPath;
    s_currentProjectName = projectName;
    return true;
}

const std::string &ProjectManager::GetCurrentProjectPath()
{
    return s_currentProjectPath;
}

const std::string &ProjectManager::GetCurrentProjectName()
{
    return s_currentProjectName;
}

const std::string &ProjectManager::GetCurrentAssetsPath()
{
    static std::string assetsPath;
    fs::path baseDir{s_currentProjectPath};

    if (!s_currentProjectName.empty() &&
        baseDir.filename().string() != s_currentProjectName)
    {
        baseDir /= s_currentProjectName;
    }

    fs::path assetDir = (baseDir / "Assets").lexically_normal();
    assetsPath = assetDir.string();
    return assetsPath;
}

